use std::path::{Path, PathBuf};

use thiserror::Error;

use crate::cli::Cli;

#[derive(Error, Debug)]
pub enum ConfigError {
    #[error("no folder at {0}")]
    LibraryPathNotFound(String),
    #[error("no metadata.db in {0}")]
    MetadataNotFound(String),
}

pub struct Config {
    pub library_path: PathBuf,
    pub metadata_path: PathBuf,
}

impl Config {
    pub fn load(args: &Cli) -> Result<Self, ConfigError> {
        let library_path = Path::new(&args.library_path).to_path_buf();

        if !library_path.exists() {
            let library_path = library_path
                .as_os_str()
                .to_str()
                .unwrap_or("<failed to parse path>")
                .to_string();
            return Err(ConfigError::LibraryPathNotFound(library_path));
        }

        let metadata_path = library_path.join("metadata.db");
        if !metadata_path.exists() {
            let metadata_path = metadata_path
                .as_os_str()
                .to_str()
                .unwrap_or("<failed to parse path>")
                .to_string();
            return Err(ConfigError::MetadataNotFound(metadata_path));
        }

        Ok(Self {
            library_path,
            metadata_path,
        })
    }
}
